package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.resources.ResourceLocation
import net.minecraft.tags.TagKey
import net.minecraft.world.item.Item
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraftforge.common.Tags
import net.minecraftforge.common.Tags.Items.COBBLESTONE
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import ru.dbotthepony.mc.otm.registryName
import java.util.function.Consumer

fun addCraftingTableRecipes(consumer: Consumer<FinishedRecipe>) {
	for ((dye, crate) in MRegistry.CARGO_CRATES.mappedColoredBlocks) {
		ShapedRecipeBuilder(crate, 1)
			.define('P', MItemTags.PLATE_TRITANIUM)
			.define('C', Tags.Items.CHESTS)
			.define('D', dye.tag)
			.pattern(" D ").pattern("PCP")
			// .unlockedBy("has_dye", has(dye.tag))
			.unlockedBy("has_tritanium_plate", has(MItemTags.PLATE_TRITANIUM))
			.unlockedBy("has_chest", has(Tags.Items.CHESTS))
			.save(consumer)

		ShapelessRecipeBuilder(crate, 1)
			.requires(Ingredient.of(MItemTags.TRITANIUM_CRATES), 1)
			.requires(Ingredient.of(dye.tag), 1)
			.unlockedBy("has_tritanium_crate", has(MItemTags.TRITANIUM_CRATES))
			.save(consumer, "${crate.registryName}_alt")
	}

	ShapedRecipeBuilder(MRegistry.CARGO_CRATES.item, 1)
		.define('P', MItemTags.PLATE_TRITANIUM)
		.define('C', Tags.Items.CHESTS)
		.pattern("PCP")
		.unlockedBy("has_tritanium_plate", has(MItemTags.PLATE_TRITANIUM))
		.unlockedBy("has_chest", has(Tags.Items.CHESTS))
		.save(consumer)

	for ((color, item) in MRegistry.TRITANIUM_BLOCK.mappedColoredItemsAll) {
		MatteryRecipe(item, 24)
			.rowAB(color?.tag, MItemTags.PLATE_TRITANIUM)
			.row(MItemTags.PLATE_TRITANIUM, COBBLESTONE, MItemTags.PLATE_TRITANIUM)
			.rowB(MItemTags.PLATE_TRITANIUM)
			.unlockedBy(MItemTags.PLATE_TRITANIUM)
			.also { if (color != null) it.unlockedBy(color.tag) }
			.build(consumer)
	}

	for ((color, item) in MRegistry.VENT.mappedColoredItemsAll) {
		MatteryRecipe(item, 24)
			.rowAB(color?.tag, MItemTags.PLATE_TRITANIUM)
			.row(MItemTags.PLATE_TRITANIUM, Items.IRON_BARS, MItemTags.PLATE_TRITANIUM)
			.rowB(MItemTags.PLATE_TRITANIUM)
			.unlockedBy(MItemTags.PLATE_TRITANIUM)
			.also { if (color != null) it.unlockedBy(color.tag) }
			.build(consumer)
	}

	for ((color, item) in MRegistry.VENT_ALTERNATIVE.mappedColoredItemsAll) {
		val other = MRegistry.VENT.mappedColoredItemsAll[color]!!
		ShapelessRecipeBuilder(item, 1).requires(other).unlockedBy(item).save(consumer)
		ShapelessRecipeBuilder(other, 1).requires(item).unlockedBy(other).save(consumer, ResourceLocation(OverdriveThatMatters.MOD_ID, "${other.registryName!!.path}_from_alt"))
	}

	for ((crate, color) in listOf(
		MItems.CRATE_RED to Tags.Items.DYES_RED,
		MItems.CRATE_BLUE to Tags.Items.DYES_BLUE,
		MItems.CRATE_YELLOW to Tags.Items.DYES_YELLOW,
		MItems.CRATE_GREEN to Tags.Items.DYES_GREEN,
		MItems.CRATE_BLACK to Tags.Items.DYES_BLACK,
		MItems.CRATE_PINK to Tags.Items.DYES_PINK,
		MItems.CRATE_PURPLE to Tags.Items.DYES_PURPLE,
	)) {
		MatteryRecipe(crate, 24)
			.rowAB(color, MItemTags.PLATE_IRON)
			.row(MItemTags.PLATE_IRON, COBBLESTONE, MItemTags.PLATE_IRON)
			.rowB(MItemTags.PLATE_IRON)
			.unlockedBy(MItemTags.PLATE_IRON)
			.unlockedBy(color)
			.build(consumer, crate.registryName!!.path + "_1")

		MatteryRecipe(crate, 24)
			.rowBC(MItemTags.PLATE_IRON, color)
			.row(MItemTags.PLATE_IRON, COBBLESTONE, MItemTags.PLATE_IRON)
			.rowB(MItemTags.PLATE_IRON)
			.unlockedBy(MItemTags.PLATE_IRON)
			.unlockedBy(color)
			.build(consumer, crate.registryName!!.path + "_2")

		MatteryRecipe(crate, 24)
			.rowB(MItemTags.PLATE_IRON)
			.row(MItemTags.PLATE_IRON, COBBLESTONE, MItemTags.PLATE_IRON)
			.rowAB(color, MItemTags.PLATE_IRON)
			.unlockedBy(MItemTags.PLATE_IRON)
			.unlockedBy(color)
			.build(consumer, crate.registryName!!.path + "_3")

		MatteryRecipe(crate, 24)
			.rowB(MItemTags.PLATE_IRON)
			.row(MItemTags.PLATE_IRON, COBBLESTONE, MItemTags.PLATE_IRON)
			.rowBC(MItemTags.PLATE_IRON, color)
			.unlockedBy(MItemTags.PLATE_IRON)
			.unlockedBy(color)
			.build(consumer, crate.registryName!!.path + "_4")
	}
}
